@@ -85,11 +85,20 @@ def refresh_style(widget):
     widget : QWidget
     """
     widgets = [widget]
-    widgets.extend(widget.findChildren(QWidget))
+
+    try:
+        widgets.extend(widget.findChildren(QWidget))
+    except:
+        # If we fail it means that widget is probably destroyed
+        return
     for child_widget in widgets:
-        child_widget.style().unpolish(child_widget)
-        child_widget.style().polish(child_widget)
-        child_widget.update()
+        try:
+            child_widget.style().unpolish(child_widget)
+            child_widget.style().polish(child_widget)
+            child_widget.update()
+        except Exception as ex:
+            # Widget was probably destroyed
+            logger.debug('Error while refreshing stylesheet. %s ', ex)
 
 
 class PyDMPrimitiveWidget(object):
@@ -195,6 +204,7 @@ class PyDMPrimitiveWidget(object):
         """
         if new_rules != self._rules:
             self._rules = new_rules
+            RulesDispatcher().unregister(self)
             try:
                 rules_list = json.loads(self._rules)
                 if rules_list:
@@ -222,7 +232,7 @@ class TextFormatter(object):
         self._user_prec = 0
         self._prec = 0
         self._unit = ""
-    
+
     def update_format_string(self):
         """
         Reconstruct the format string to be used when representing the
@@ -240,7 +250,7 @@ class TextFormatter(object):
         if self._show_units and self._unit != "":
             self.format_string += " {}".format(self._unit)
         return self.format_string
-    
+
     def precision_changed(self, new_precision):
         """
         Callback invoked when the Channel has new precision value.
@@ -256,7 +266,7 @@ class TextFormatter(object):
             self._prec = new_precision
             if self.value is not None:
                 self.value_changed(self.value)
-    
+
     @Slot(int)
     @Slot(float)
     def precisionChanged(self, new_prec):
@@ -270,7 +280,7 @@ class TextFormatter(object):
         new_prec : int or float
         """
         self.precision_changed(new_prec)
-    
+
     @Property(int)
     def precision(self):
         """
@@ -304,7 +314,7 @@ class TextFormatter(object):
             self._user_prec = int(new_prec)
             if not is_qt_designer() or config.DESIGNER_ONLINE:
                 self.value_changed(self.value)
-    
+
     @Slot(str)
     def unitChanged(self, new_unit):
         """
@@ -317,7 +327,7 @@ class TextFormatter(object):
         new_unit : str
         """
         self.unit_changed(new_unit)
-    
+
     def unit_changed(self, new_unit):
         """
         Callback invoked when the Channel has new unit value.
@@ -333,7 +343,7 @@ class TextFormatter(object):
             self._unit = new_unit
             if self.value is not None:
                 self.value_changed(self.value)
-    
+
 
     @Property(bool)
     def showUnits(self):
@@ -370,7 +380,7 @@ class TextFormatter(object):
         if self._show_units != show_units:
             self._show_units = show_units
             self.update_format_string()
-    
+
     @Property(bool)
     def precisionFromPV(self):
         """
@@ -422,7 +432,7 @@ class TextFormatter(object):
         if self._precision_from_pv is None or self._precision_from_pv != bool(value):
             self._precision_from_pv = value
             self.update_format_string()
-    
+
     def value_changed(self, new_val):
         """
         Callback invoked when the Channel value is changed.
@@ -435,7 +445,7 @@ class TextFormatter(object):
         super(TextFormatter, self).value_changed(new_val)
         self.update_format_string()
 
-  
+
 class PyDMWidget(PyDMPrimitiveWidget):
     """
     PyDM base class for Read-Only widgets.
@@ -901,6 +911,9 @@ class PyDMWidget(PyDMPrimitiveWidget):
                 self._channels.remove(channel)
             # Load new channel
             self._channel = str(value)
+            if not self._channel:
+                logger.debug('Channel was set to an empty string.')
+                return
             channel = PyDMChannel(address=self._channel,
                                   connection_slot=self.connectionStateChanged,
                                   value_slot=self.channelValueChanged,
